'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _child_process = require('child_process');

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _fs = require('fs');

var _hostile = require('hostile');

var _hostile2 = _interopRequireDefault(_hostile);

var _os = require('os');

var _path = require('path');

var _updateNotifier = require('update-notifier');

var _updateNotifier2 = _interopRequireDefault(_updateNotifier);

var _package = require('../package.json');

var _package2 = _interopRequireDefault(_package);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

(0, _updateNotifier2.default)({ pkg: _package2.default }).notify();

// load existing config if any

var configFilePath = (0, _path.join)((0, _os.homedir)(), 'ffocus.json');
var config = void 0;
try {
  config = require(configFilePath);
} catch (e) {
  config = {};
}
var presets = config.presets || {};

// register cleanups
process.on('SIGINT', cleanup);
process.on('uncaughtException', cleanup);

// program

log('Welcome to ffocus 👋');

_commander2.default.usage('[minutes] [host1 host2 host3 ...]').option('-a, --add <host>', 'adds a new host').option('-r, --remove <host>', 'removes an existing host').option('-t, --duration <minutes>', 'sets the duration for a preset').option('-p, --preset <name>', 'a preset').option('-l, --list', 'lists all the presets').version(_package2.default.version);

_commander2.default.parse(process.argv);

// list presets

if (_commander2.default.list) {
  var availablePresets = Object.keys(presets);
  if (!availablePresets.length) {
    log('📚  No presets found.');
  } else {
    log('📚  Available presets:');
    Object.keys(presets).forEach(function (preset) {
      log('\uD83D\uDCDA  "' + preset + '" runs every ' + presets[preset].minutes + ' minutes');
      log('\uD83D\uDCDA  \tblocked sites:');
      presets[preset].sites.forEach(function (site) {
        return log('\uD83D\uDCDA  \t\t' + site);
      });
    });
  }
  process.exit(0);
}

var minutes = Number(_commander2.default.args[0]) || null;
var sites = _commander2.default.args.length > 1 ? _commander2.default.args.slice(1) : null;

// parse action (add, remove or duration)

var action = ['add', 'remove', 'duration'].reduce(function (acc, flag) {
  var action = _commander2.default[flag];
  if (action) {
    acc = {
      name: flag,
      value: action
    };
  }
  return acc;
}, null);

if (action) {
  (function () {
    // actions must have a preset
    if (!_commander2.default.preset) {
      error('--' + action.name + ' should be paired with --preset <name>');
    }

    var dummyPreset = {
      sites: [],
      minutes: 60
    };

    var actions = {
      add: function add(preset, host) {
        var original = presets[preset] || dummyPreset;
        if (original.sites.indexOf(host) !== -1) {
          log('"' + preset + '" already contains ' + host);
          return original;
        }
        log('Adding ' + host + ' to "' + preset + '"');
        return _extends({}, original, {
          sites: original.sites.concat([host])
        });
      },
      remove: function remove(preset, host) {
        log('Removing ' + host + ' from "' + preset + '"');
        var original = presets[preset] || dummyPreset;
        return _extends({}, original, {
          sites: original.sites.filter(function (h) {
            return h !== host;
          })
        });
      },
      duration: function duration(preset, minutes) {
        log('Setting the duration for "' + preset + '" to ' + minutes + ' minutes');
        var mins = Number(minutes);
        if (isNaN(mins)) {
          error('"minutes" must be a number');
        }
        var original = presets[preset] || dummyPreset;
        return _extends({}, original, {
          minutes: mins
        });
      }
    };

    var update = actions[action.name](_commander2.default.preset, action.value);

    presets = Object.assign({}, presets, _defineProperty({}, _commander2.default.preset, update));

    if (!presets[_commander2.default.preset]) {
      error('preset "' + _commander2.default.preset + '" not found');
    }

    config = Object.assign({}, config, { presets: presets });

    write(config, function (err) {
      if (err) {
        error('An error occurred while saving the configuration: ' + err);
      }
      log('Configuration saved.');
    });
    process.exit(0);
  })();
}

// run a preset or one-off ffocus section

var preset = void 0;
if (_commander2.default.preset) {
  // preset
  preset = presets[_commander2.default.preset];
  if (!preset) {
    error('preset "' + _commander2.default.preset + '" not found');
  }
} else {
  // one off
  if (!minutes || !sites) {
    _commander2.default.help();
  }
  preset = {
    minutes: minutes,
    sites: sites
  };
}

run(preset);

// helpers

function run(_ref) {
  var minutes = _ref.minutes,
      sites = _ref.sites;

  if (!sites.length) {
    log('No sites to block.');
    process.exit(0);
  }

  sites.forEach(function (site) {
    try {
      _hostile2.default.set('127.0.0.1', site);
      log('\uD83D\uDEAB  Blocking ' + site);
    } catch (err) {
      error(err.message, true);
    }
  });

  setTimeout(function () {
    log('\uD83D\uDC4A  You\'re doing great!');
    log('\uD83D\uDC4A  You\'ve been more productive for ' + minutes / 2 + ' minutes');
  }, minutes / 2 * 6e4);

  setTimeout(function () {
    log('\uD83D\uDE4F  Congrats you managed to focus for ' + minutes + ' minutes!');
    cleanup();
  }, minutes * 6e4);

  log('\u2705  Focus for ' + minutes + ' minutes.');
}

function write(data, cb) {
  (0, _fs.writeFileSync)(configFilePath, JSON.stringify(data, null, 2), { flag: 'w' }, 'utf8', cb || function () {});
}

function cleanup() {
  var allSites = Object.keys(config.presets || {}).reduce(function (acc, preset) {
    acc = acc.concat(presets[preset].sites);
    return acc;
  }, []).concat(sites).filter(Boolean);

  allSites.forEach(function (site) {
    try {
      _hostile2.default.remove('127.0.0.1', site);
    } catch (err) {
      error(err.message, true);
    }
  });

  process.exit(0);
}

function log(msg) {
  console.log.call(console, '\uD83D\uDD0D  ' + msg);
}

function error(msg, sudo) {
  console.error.call(console, '\u2620\uFE0F  ' + msg);
  sudo && console.error('☠️  ffocus need sudo rights to run');
  process.exit(1);
}
